"""
Plot histograms and distribution of barge lengths and bunkering time
Created: 11. April 2018
Author: Marion Aarsnes
"""

import pandas as pd                 # Read CSV
import seaborn as sns               # Library of plots built upon matplotlib
import matplotlib.pyplot as plt     # Plots
import matplotlib.ticker as mtick   # Formatting plots
import numpy as np                  # Scientific computing
from matplotlib.pyplot import xticks   # Rotate plot

# Read CSV file
AIS_dataframe = pd.read_csv('filedirectory/filename.csv')
print(AIS_dataframe.columns)

# Clean up dataframe
AIS_dataframe = AIS_dataframe[['imo.b', 'shipType.s', 'imo.s', 'length.s', 'length.b', 'bunkeringLength_hrs', 'anchoringLength_hrs', 'bunkeringType', 'confidence_mean']]
AIS_dataframe = AIS_dataframe.rename(columns={"imo.b": "IMO",
                                              "shipType.s": "ShipType",
                                              "imo.s": "IMOship",
                                              "length.s": "ShipLength",
                                              "length.b": "BargeLength",
                                              "anchoringLength_hrs": "AnchoringLengthHrs",
                                              "bunkeringLength_hrs": "BunkeringLengthHrs",
                                              "bunkeringType": "BunkeringType",
                                              "confidence_mean": "ConfMean"})

AIS_dataframe = AIS_dataframe.fillna(value="nan")[AIS_dataframe.ShipType != "nan"]
# Rename ship types
AIS_dataframe = AIS_dataframe.replace({"FSO (Floating, Storage, Offloading)": "FSO", "FPSO (Floating, Production, Storage, Offloading)": "FPSO"})
# Requiring a minimum confidence mean of 0.7
AIS_dataframe = AIS_dataframe[AIS_dataframe.ConfMean >= 0.7]


# Setting plot style
sns.set(style="whitegrid", color_codes=True)

for shiptype in AIS_dataframe['ShipType'].unique():
    print(shiptype)
    print(AIS_dataframe[AIS_dataframe.ShipType == shiptype]['ShipType'].count())

# Reduce dataframe to shiptypes with minimum 100 registrations
AIS_dataframe = AIS_dataframe.query("ShipType in ('Bulk Carrier','Container Ship','Crude Oil Tanker','Chemical/Oil Products Tanker',"
                                    "'Vehicles Carrier','LPG Tanker','General Cargo Ship','Oil Products Tanker',"
                                    "'Ore Carrier','LNG Tanker',)")

# ----------------- Distribution of bunker barge lengths ------------------

# Define number of bins
# Bin size
res_blen = 2

# Calculate bin number based on bin size
min_blen = min(AIS_dataframe['BargeLength'])
max_blen = max(AIS_dataframe['BargeLength'])
bsize_blen = int((max_blen-min_blen)/res_blen)

# Plot histogram
ax = sns.distplot(AIS_dataframe['BargeLength'],
                      bins=(bsize_blen+3),kde=False, hist_kws=dict(edgecolor="b", linewidth=1))
ax.set(ylabel="Number of registered barge lengths", xlabel="Barge length")
plt.xlim(0,130)
plt.legend()
plt.show()

# Plot histogram with Kernel Density Line
ax = sns.distplot(AIS_dataframe['BargeLength'],
                  bins=bsize_blen, hist_kws=dict(edgecolor="b", linewidth=1))
ax.set(ylabel="Density", xlabel="Barge length")
plt.xlim(0,130)
plt.legend()
plt.show()

# ---------------- Distribution of bunkering time for all bunkering types -------------------
# Requiring a maximum bunkering time of 24 hours
AIS_dataframe = AIS_dataframe[AIS_dataframe.BunkeringLengthHrs <= 24]

#Define bin size
res_blenh = 1
min_blenh = min(AIS_dataframe['BunkeringLengthHrs'])
max_blenh = max(AIS_dataframe['BunkeringLengthHrs'])
bsize_blenh = int((max_blenh-min_blenh)/res_blenh) # bin number must be an integer

# Plot histogram
ax = sns.distplot(AIS_dataframe['BunkeringLengthHrs'],
                      bins=bsize_blenh, kde=False, hist_kws=dict(edgecolor="b", linewidth=1))
ax.set(ylabel="Number of registered time lengths", xlabel="Bunkering time [hours]")
plt.xlim(0,35)
plt.legend()
plt.show()

# Plot histogram with Kernel Density Line
ax = sns.distplot(AIS_dataframe['BunkeringLengthHrs'],
                  bins=bsize_blenh, hist_kws=dict(edgecolor="b", linewidth=1))
ax.set(ylabel="Density", xlabel="Bunkering time [hours]")
plt.xlim(0,35)
plt.legend()
plt.show()


# ---------------- Distribution of bunkering time at anchorage and berth -----------------

AIS_dataframe = AIS_dataframe[AIS_dataframe.BunkeringLengthHrs <= 24]

# Creating new dataframes
# Anchorage
AIS_dataframe_anchor = AIS_dataframe[AIS_dataframe.BunkeringType == 'Anchorage']
# Berth
AIS_dataframe_berth = AIS_dataframe[AIS_dataframe.BunkeringType == 'Berth']

# Define number of bins
# Bin size
res_blenab = 1

# Number of bins at anchor
min_blena = min(AIS_dataframe_anchor['BunkeringLengthHrs'])
max_blena = max(AIS_dataframe_anchor['BunkeringLengthHrs'])
bsize_blena = int(((max_blena-min_blena)/res_blenab))

# Number of bins at berth
min_blenb = min(AIS_dataframe_berth['BunkeringLengthHrs'])
max_blenb = max(AIS_dataframe_berth['BunkeringLengthHrs'])
bsize_blenb = int(((max_blenb-min_blenb)/res_blenab))

ax = sns.distplot(AIS_dataframe_anchor['BunkeringLengthHrs'],
                  bins=bsize_blena, hist_kws=dict(edgecolor="b", linewidth=1))
ax.set(ylabel="Density", xlabel="Bunkering time at anchorage [hours]")
plt.xlim(0,27)
plt.legend()
plt.show()

ax = sns.distplot(AIS_dataframe_berth['BunkeringLengthHrs'],
                  bins=bsize_blenb, hist_kws=dict(edgecolor="b", linewidth=1))
ax.set(ylabel="Density", xlabel="Bunkering time at berth [hours]")
plt.xlim(0,27)
plt.legend()
plt.show()
