"""
Script for gathering values exceeding a certain length
E.g. All anchorage length of more than 150 hours.
Plot lay time and bunkering time.
Created: 23.april 2018
Author: Marion Aarsnes
"""

import pandas as pd                 # Read CSV
import seaborn as sns               # Library of plots built upon matplotlib
import matplotlib.pyplot as plt     # Plots

# Read CSV file
AIS_dataframe = pd.read_csv('filedirectory/filename.csv')
print(AIS_dataframe.columns)
# Num lines: 19 901

# Clean up dataframe
AIS_dataframe = AIS_dataframe[['imo.b', 'shipType.s', 'length.s', 'bunkeringLength_hrs', 'anchoringLength_hrs', 'bunkeringType']]
AIS_dataframe = AIS_dataframe.rename(columns={"imo.b": "IMO",
                                              "shipType.s": "ShipType",
                                              "length.s": "ShipLength",
                                              "anchoringLength_hrs": "AnchoringLengthHrs",
                                              "bunkeringLength_hrs": "BunkeringLengthHrs",
                                              "bunkeringType": "BunkeringType"})



AIS_dataframe = AIS_dataframe.fillna(value="nan")[AIS_dataframe.ShipType != "nan"]


# -------------------- Anchorage lasting longer than value a -----------------------

# Creating new dataframe for longer lasting anchorages
a = 80
b = 24

# Creating one value for all operations lasting longer than value a
def fill_values(a):
    anch_len_x = []
    bunk_len_x = []
    for i in range(len(AIS_dataframe.AnchoringLengthHrs)):
        anch_len_x.append(a)
        bunk_len_x.append(b)
    return anch_len_x, bunk_len_x

anch_len_x, bunk_len_x = fill_values(a)


# Creating column index to dataframe
AIS_dataframe['AnchorLengthx'] = AIS_dataframe.index
AIS_dataframe['BunkLengthx'] = AIS_dataframe.index
# Inserting new data to dataframe
AIS_dataframe.loc[:,'AnchorLengthx'] = pd.Series(anch_len_x, index=AIS_dataframe.index)
AIS_dataframe.loc[:,'BunkLengthx'] = pd.Series(bunk_len_x, index=AIS_dataframe.index)


# ------------------------------- Length grouping ---------------------------------

# Require a minimum confidence mean of 0.7
print(AIS_dataframe['ShipType'].unique())

# Reduce dataframe to shiptypes with minimum 100 registrations
AIS_dataframe = AIS_dataframe.query("ShipType in ('Bulk Carrier','Container Ship','Crude Oil Tanker','Chemical/Oil Products Tanker',"
                                    "'Vehicles Carrier','LPG Tanker','General Cargo Ship','Oil Products Tanker',"
                                    "'Ore Carrier','LNG Tanker',)")

# Creating new dataframe for operations lasting longer than value a
AIS_dataframe_x = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs >= a]

AIS_dataframe_n = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs < a]

# Creating new dataframes for bunkering type
# Anchorage
AIS_dataframe_anchor = AIS_dataframe_n[AIS_dataframe_n.BunkeringType == 'Anchorage']
# Berth
AIS_dataframe_berth = AIS_dataframe_n[AIS_dataframe_n.BunkeringType == 'Berth']


# ---------------Define number of bins---------------

# Bin size
res_anch = 2
res_bunk = 1
# Ship length grouping
length = 150


# Anchor
min_asm = min(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength<=length]['AnchoringLengthHrs'])
max_asm = max(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength<=length]['AnchoringLengthHrs'])
bsize_asm = int((max_asm-min_asm)/res_anch) # bin size must be an integer
min_ala = min(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength>length]['AnchoringLengthHrs'])
max_ala = max(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength>length]['AnchoringLengthHrs'])
bsize_ala = int((max_ala-min_ala)/res_anch) # bin size must be an integer

# Berth
min_bsm = min(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength<=length]['AnchoringLengthHrs'])
max_bsm = max(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength<=length]['AnchoringLengthHrs'])
bsize_bsm = int((max_bsm-min_bsm)/res_anch) # bin size must be an integer
min_bla = min(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength>length]['AnchoringLengthHrs'])
max_bla = max(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength>length]['AnchoringLengthHrs'])
bsize_bla = int((max_bla-min_bla)/res_anch) # bin size must be an integer

# Bunkering at anchorage
min_asmb = min(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength<=length]['BunkeringLengthHrs'])
max_asmb = max(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength<=length]['BunkeringLengthHrs'])
bsize_asmb = int((max_asmb-min_asmb)/res_bunk) # bin size must be an integer
min_alab = min(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength>length]['BunkeringLengthHrs'])
max_alab = max(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength>length]['BunkeringLengthHrs'])
bsize_alab = int((max_alab-min_alab)/res_bunk) # bin size must be an integer

# Bunkering at berth
min_bsmb = min(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength<=length]['BunkeringLengthHrs'])
max_bsmb = max(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength<=length]['BunkeringLengthHrs'])
bsize_bsmb = int((max_bsmb-min_bsmb)/res_bunk) # bin size must be an integer
min_blab = min(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength>length]['BunkeringLengthHrs'])
max_blab = max(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength>length]['BunkeringLengthHrs'])
bsize_blab = int((max_blab-min_blab)/res_bunk) # bin size must be an integer


# ----------------- Plot laying and bunkering time for operations --------------------

#sns.set()
sns.set(style="whitegrid", color_codes=True)

# Anchorage
plt.figure()
ax = sns.distplot(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength<=150]['AnchoringLengthHrs'],
                     bins=bsize_asm, color='b', hist_kws=dict(edgecolor="b", linewidth=1), label='Shiplength <= ' + str(length) + ' meters') #kde=True, color="b",
ax2 = sns.distplot(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength>150]['AnchoringLengthHrs'],
                     bins=bsize_ala, color="r", hist_kws=dict(edgecolor="r", linewidth=1), label='Shiplength > ' + str(length) + ' meters') # kde=True, color="r",
axa = sns.distplot(AIS_dataframe_x[AIS_dataframe_x.BunkeringType == 'Anchorage']['AnchorLengthx'],
                     kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Anchorage of more than ' + str(a) + ' hours')
#ax.axvline(3, color="k", linestyle="--")
ax.set(ylabel="Ship density", xlabel="Anchorage time [hours]", title ='Anchorage time')
plt.xlim(0,a+5)
#plt.ylim(0,0.5)
plt.legend()
plt.show()

# Berth
plt.figure()
ax3 = sns.distplot(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength<=150]['AnchoringLengthHrs'],
                     bins=bsize_bsm, color='b', hist_kws=dict(edgecolor="b", linewidth=1), label='Shiplength <= ' + str(length) + ' meters') # kde=False, color="b",
ax5 = sns.distplot(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength>150]['AnchoringLengthHrs'],
                     bins=bsize_bla, color="r", hist_kws=dict(edgecolor="r", linewidth=1), label='Shiplength > ' + str(length) + ' meters') # kde=False, color="r",
axb = sns.distplot(AIS_dataframe_x[AIS_dataframe_x.BunkeringType == 'Berth']['AnchorLengthx'],
                     kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Anchorage of more than ' + str(a) + ' hours')
#ax3.axvline(3, color="k", linestyle="--")
ax3.set(ylabel="Ship density", xlabel="Berth time [hours]", title ='Berth time')
plt.xlim(0,a+5)
plt.legend()
plt.show()


# Bunkering time at anchorage
plt.figure()
ax0 = sns.distplot(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength<=150]['BunkeringLengthHrs'],
                     bins=bsize_asmb, color='b', hist_kws=dict(edgecolor="b", linewidth=1), label='Shiplength <= ' + str(length) + ' meters') # kde=False, color="b",
ax2 = sns.distplot(AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength>150]['BunkeringLengthHrs'],
                     bins=bsize_alab, color="r", hist_kws=dict(edgecolor="r", linewidth=1), label='Shiplength > ' + str(length) + ' meters') # kde=False, color="r",
#axc = sns.distplot(AIS_dataframe_x[AIS_dataframe_x.BunkeringType == 'Anchorage']['BunkLengthx'],
                     #kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Anchorage of more than 180 hours')
#ax0.axvline(3, color="k", linestyle="--")
ax0.set(ylabel="Ship density", xlabel="Bunkering time [hours]", title ='Bunkering time at anchorage')
plt.xlim(0,b+5)
plt.legend()
plt.show()


# Bunkering time at berth
plt.figure()
ax3 = sns.distplot(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength<=150]['BunkeringLengthHrs'],
                     bins=bsize_bsmb, color='b', hist_kws=dict(edgecolor="b", linewidth=1), label='Shiplength <= ' + str(length) + ' meters') # kde=False, color="b",
ax5 = sns.distplot(AIS_dataframe_berth[AIS_dataframe_berth.ShipLength>150]['BunkeringLengthHrs'],
                     bins=bsize_blab, color="r", hist_kws=dict(edgecolor="r", linewidth=1), label='Shiplength > ' + str(length) + ' meters') # kde=False, color="r",
#axd = sns.distplot(AIS_dataframe_x[AIS_dataframe_x.BunkeringType == 'Berth']['BunkLengthx'],
                     #kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Anchorage of more than 180 hours')
#ax3.axvline(3, color="k", linestyle="--")
ax5.set(ylabel="Ship density", xlabel="Bunkering time [hours]", title ='Bunkering time at berth')
plt.xlim(0,b+5)
plt.legend()
plt.show()


# ----------------- Normalised plots --------------------

# Creating new dataframe for operations lasting longer than value a
AIS_dataframe_x = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs >= a]

AIS_dataframe_n = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs < a]

# Creating new dataframes for bunkering type
# Anchorage
AIS_dataframe_anchor = AIS_dataframe_n[AIS_dataframe_n.BunkeringType == 'Anchorage']
# Berth
AIS_dataframe_berth = AIS_dataframe_n[AIS_dataframe_n.BunkeringType == 'Berth']

# ------- All operations --------

# Create new dataframes
AIS_dataframe_above150 = AIS_dataframe_n[AIS_dataframe_n.ShipLength > 150]
AIS_dataframe_below150 = AIS_dataframe_n[AIS_dataframe_n.ShipLength <= 150]

# Sort dataframes
AIS_dataframe_above150l = AIS_dataframe_above150.sort_values('AnchoringLengthHrs')
AIS_dataframe_below150l = AIS_dataframe_below150.sort_values('AnchoringLengthHrs')
AIS_dataframe_above150b = AIS_dataframe_above150.sort_values('BunkeringLengthHrs')
AIS_dataframe_below150b = AIS_dataframe_below150.sort_values('BunkeringLengthHrs')

# Create list numerating operations
# Create new column with normalised values
def fill_value_allabove150l():
    norm_allabove150l = []
    for i in range(len(AIS_dataframe_above150l.ShipLength)):
        norm_allabove150l.append((i+1)/len(AIS_dataframe_above150l.ShipLength)*100)
    return norm_allabove150l

norm_allabove150l = fill_value_allabove150l()

def fill_value_allbelow150l():
    norm_allbelow150l = []
    for i in range(len(AIS_dataframe_below150l.ShipLength)):
        norm_allbelow150l.append((i+1)/len(AIS_dataframe_below150l.ShipLength)*100)
    return norm_allbelow150l

norm_allbelow150l = fill_value_allbelow150l()

def fill_value_allabove150b():
    norm_allabove150b = []
    for i in range(len(AIS_dataframe_above150b.ShipLength)):
        norm_allabove150b.append((i+1)/len(AIS_dataframe_above150b.ShipLength)*100)
    return norm_allabove150b

norm_allabove150b = fill_value_allabove150b()

def fill_value_allbelow150b():
    norm_allbelow150b = []
    for i in range(len(AIS_dataframe_below150b.ShipLength)):
        norm_allbelow150b.append((i+1)/len(AIS_dataframe_below150b.ShipLength)*100)
    return norm_allbelow150b

norm_allbelow150b = fill_value_allbelow150b()

# Append normalised column
AIS_dataframe_above150l['norm'] = norm_allabove150l
AIS_dataframe_below150l['norm'] = norm_allbelow150l
AIS_dataframe_above150b['norm'] = norm_allabove150b
AIS_dataframe_below150b['norm'] = norm_allbelow150b


# Plot laying time
plt.plot(AIS_dataframe_above150l.AnchoringLengthHrs, AIS_dataframe_above150l.norm, color='r', label='Shiplength > ' + str(length) + ' meters')
plt.plot(AIS_dataframe_below150l.AnchoringLengthHrs, AIS_dataframe_below150l.norm, color='b', label='Shiplength <= ' + str(length) + ' meters')
plt.title('Laying time normalised over number of operations')
plt.xlabel("Laying time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,a+5)
plt.legend()
plt.show()

# Plot bunkering time
plt.plot(AIS_dataframe_above150b.BunkeringLengthHrs, AIS_dataframe_above150b.norm, color='r', label='Shiplength > ' + str(length) + ' meters')
plt.plot(AIS_dataframe_below150b.BunkeringLengthHrs, AIS_dataframe_below150b.norm, color='b', label='Shiplength <= ' + str(length) + ' meters')
plt.title('Bunkering time normalised over number of operations')
plt.xlabel("Bunkering time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,b+5)
plt.legend()
plt.show()



# ------- At anchorage --------
AIS_dataframe_anchor_above150 = AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength > 150]
AIS_dataframe_anchor_below150 = AIS_dataframe_anchor[AIS_dataframe_anchor.ShipLength <= 150]

# Sort dataframes
AIS_dataframe_anchor_above150l = AIS_dataframe_anchor_above150.sort_values('AnchoringLengthHrs')
AIS_dataframe_anchor_below150l = AIS_dataframe_anchor_below150.sort_values('AnchoringLengthHrs')
AIS_dataframe_anchor_above150b = AIS_dataframe_anchor_above150.sort_values('BunkeringLengthHrs')
AIS_dataframe_anchor_below150b = AIS_dataframe_anchor_below150.sort_values('BunkeringLengthHrs')

# Create list numerating operations
# Create new column with normalised values
def fill_value_anchor_allabove150l():
    norm_anchor_allabove150l = []
    for i in range(len(AIS_dataframe_anchor_above150l.ShipLength)):
        norm_anchor_allabove150l.append((i+1)/len(AIS_dataframe_anchor_above150l.ShipLength)*100)
    return norm_anchor_allabove150l

norm_anchor_allabove150l = fill_value_anchor_allabove150l()

def fill_value_anchor_allbelow150l():
    norm_anchor_allbelow150l = []
    for i in range(len(AIS_dataframe_anchor_below150l.ShipLength)):
        norm_anchor_allbelow150l.append((i+1)/len(AIS_dataframe_anchor_below150l.ShipLength)*100)
    return norm_anchor_allbelow150l

norm_anchor_allbelow150l = fill_value_anchor_allbelow150l()

def fill_value_anchor_allabove150b():
    norm_anchor_allabove150b = []
    for i in range(len(AIS_dataframe_anchor_above150b.ShipLength)):
        norm_anchor_allabove150b.append((i+1)/len(AIS_dataframe_anchor_above150b.ShipLength)*100)
    return norm_anchor_allabove150b

norm_anchor_allabove150b = fill_value_anchor_allabove150b()

def fill_value_anchor_allbelow150b():
    norm_anchor_allbelow150b = []
    for i in range(len(AIS_dataframe_anchor_below150b.ShipLength)):
        norm_anchor_allbelow150b.append((i+1)/len(AIS_dataframe_anchor_below150b.ShipLength)*100)
    return norm_anchor_allbelow150b

norm_anchor_allbelow150b = fill_value_anchor_allbelow150b()

# Append normalised column
AIS_dataframe_anchor_above150l['norm'] = norm_anchor_allabove150l
AIS_dataframe_anchor_below150l['norm'] = norm_anchor_allbelow150l
AIS_dataframe_anchor_above150b['norm'] = norm_anchor_allabove150b
AIS_dataframe_anchor_below150b['norm'] = norm_anchor_allbelow150b


# Plot laying time
plt.plot(AIS_dataframe_anchor_above150l.AnchoringLengthHrs, AIS_dataframe_anchor_above150l.norm, color='r', label='Shiplength > ' + str(length) + ' meters')
plt.plot(AIS_dataframe_anchor_below150l.AnchoringLengthHrs, AIS_dataframe_anchor_below150l.norm, color='b', label='Shiplength <= ' + str(length) + ' meters')
plt.title('Anchorage time normalised over number of operations')
plt.xlabel("Laying time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,a+5)
plt.legend()
plt.show()

# Plot bunkering time
plt.plot(AIS_dataframe_anchor_above150b.BunkeringLengthHrs, AIS_dataframe_anchor_above150b.norm, color='r', label='Shiplength > ' + str(length) + ' meters')
plt.plot(AIS_dataframe_anchor_below150b.BunkeringLengthHrs, AIS_dataframe_anchor_below150b.norm, color='b', label='Shiplength <= ' + str(length) + ' meters')
plt.title('Bunkering time normalised over number of operations at anchorage')
plt.xlabel("Bunkering time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,b+5)
plt.legend()
plt.show()


# # ------- At berth --------
AIS_dataframe_berth_above150 = AIS_dataframe_berth[AIS_dataframe_berth.ShipLength > 150]
AIS_dataframe_berth_below150 = AIS_dataframe_berth[AIS_dataframe_berth.ShipLength <= 150]

# Sort dataframes
AIS_dataframe_berth_above150l = AIS_dataframe_berth_above150.sort_values('AnchoringLengthHrs')
AIS_dataframe_berth_below150l = AIS_dataframe_berth_below150.sort_values('AnchoringLengthHrs')
AIS_dataframe_berth_above150b = AIS_dataframe_berth_above150.sort_values('BunkeringLengthHrs')
AIS_dataframe_berth_below150b = AIS_dataframe_berth_below150.sort_values('BunkeringLengthHrs')

# Create list numerating operations
# Create new column with normalised values
def fill_value_berth_allabove150l():
    norm_berth_allabove150l = []
    for i in range(len(AIS_dataframe_berth_above150l.ShipLength)):
        norm_berth_allabove150l.append((i+1)/len(AIS_dataframe_berth_above150l.ShipLength)*100)
    return norm_berth_allabove150l

norm_berth_allabove150l = fill_value_berth_allabove150l()

def fill_value_berth_allbelow150l():
    norm_berth_allbelow150l = []
    for i in range(len(AIS_dataframe_berth_below150l.ShipLength)):
        norm_berth_allbelow150l.append((i+1)/len(AIS_dataframe_berth_below150l.ShipLength)*100)
    return norm_berth_allbelow150l

norm_berth_allbelow150l = fill_value_berth_allbelow150l()

def fill_value_berth_allabove150b():
    norm_berth_allabove150b = []
    for i in range(len(AIS_dataframe_berth_above150b.ShipLength)):
        norm_berth_allabove150b.append((i+1)/len(AIS_dataframe_berth_above150b.ShipLength)*100)
    return norm_berth_allabove150b

norm_berth_allabove150b = fill_value_berth_allabove150b()

def fill_value_berth_allbelow150b():
    norm_berth_allbelow150b = []
    for i in range(len(AIS_dataframe_berth_below150b.ShipLength)):
        norm_berth_allbelow150b.append((i+1)/len(AIS_dataframe_berth_below150b.ShipLength)*100)
    return norm_berth_allbelow150b

norm_berth_allbelow150b = fill_value_berth_allbelow150b()

# Append normalised column
AIS_dataframe_berth_above150l['norm'] = norm_berth_allabove150l
AIS_dataframe_berth_below150l['norm'] = norm_berth_allbelow150l
AIS_dataframe_berth_above150b['norm'] = norm_berth_allabove150b
AIS_dataframe_berth_below150b['norm'] = norm_berth_allbelow150b


# Plot laying time
plt.plot(AIS_dataframe_berth_above150l.AnchoringLengthHrs, AIS_dataframe_berth_above150l.norm, color='r', label='Shiplength > ' + str(length) + ' meters')
plt.plot(AIS_dataframe_berth_below150l.AnchoringLengthHrs, AIS_dataframe_berth_below150l.norm, color='b', label='Shiplength <= ' + str(length) + ' meters')
plt.title('Berth time normalised over number of operation')
plt.xlabel("Laying time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,a+5)
plt.legend()
plt.show()

# Plot bunkering time
plt.plot(AIS_dataframe_berth_above150b.BunkeringLengthHrs, AIS_dataframe_berth_above150b.norm, color='r', label='Shiplength > ' + str(length) + ' meters')
plt.plot(AIS_dataframe_berth_below150b.BunkeringLengthHrs, AIS_dataframe_berth_below150b.norm, color='b', label='Shiplength <= ' + str(length) + ' meters')
plt.title('Bunkering time normalised over number of operations at berth')
plt.xlabel("Bunkering time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,b+5)
plt.legend()
plt.show()