"""
Plot Waiting Time and Post Bunkering Time normalised over number of bunker operations
Created: 05. May 2018
Author: Marion Aarsnes
"""

# Import packages
import pandas as pd                 # Read CSV
import numpy as np                  # Scientific calculation
import seaborn as sns               # Library of plots built upon matplotlib
import matplotlib.pyplot as plt     # Plots
import matplotlib.ticker as mtick   # Format plots

# Read CSV file
AIS_dataframe = pd.read_csv('/Users/marionaarsnes/Documents/Masteroppgave/xxfilesforcodes/VPS2017_BunkerMatch_2018_02_28.csv')
print(AIS_dataframe.columns)

# Clean up dataframe
AIS_dataframe = AIS_dataframe[['imo.b', 'shipType.s', 'length.s', 'bunkeringLength_hrs', 'anchoringLength_hrs', 'BunkUnixStart',
       'BunkUnixEnd', 'AnchorUnixStart', 'AnchorUnixEnd', 'confidence_mean', 'bunkeringType', 'timeBeforeBunkering_hrs', 'timeAfterBunkering_hrs']]
AIS_dataframe = AIS_dataframe.rename(columns={"imo.b": "IMO",
                                              "shipType.s": "ShipType",
                                              "length.s": "ShipLength",
                                              "anchoringLength_hrs": "AnchoringLengthHrs",
                                              "bunkeringLength_hrs": "BunkeringLengthHrs",
                                              "confidence_mean": "ConfMean",
                                              "bunkeringType": "BunkeringType",
                                              "timeBeforeBunkering_hrs": "WaitingTime",
                                              "timeAfterBunkering_hrs": "PBTime"})

AIS_dataframe = AIS_dataframe.fillna(value="nan")[AIS_dataframe.ShipType != "nan"]
# Remove rows with Confidence Mean < 0.7
AIS_dataframe = AIS_dataframe[AIS_dataframe.ConfMean >= 0.7]

# Reduce dataframe to shiptypes with minimum 100 registrations
AIS_dataframe = AIS_dataframe.query("ShipType in ('Bulk Carrier','Container Ship','Crude Oil Tanker','Chemical/Oil Products Tanker',"
                                    "'Vehicles Carrier','LPG Tanker','General Cargo Ship','Oil Products Tanker',"
                                    "'Ore Carrier','LNG Tanker',)")

# Set plot style
sns.set(style="whitegrid", color_codes=True)

# Create new dataframes for bunkering type
# Anchorage
AIS_dataframe_anchor = AIS_dataframe[AIS_dataframe.BunkeringType == 'Anchorage']
# Berth
AIS_dataframe_berth = AIS_dataframe[AIS_dataframe.BunkeringType == 'Berth']

# Create new column with normalised values
def fill_value():
    norm = []
    for i in range(len(AIS_dataframe.ConfMean)):
        norm.append((i+1)/len(AIS_dataframe.ConfMean)*100)
    return norm

norm = fill_value()

# ------------------Normalised Waiting Time (WT) ---------------------

AIS_dataframe_wt = AIS_dataframe

# Sorting by increasing certainty
AIS_dataframe_wt = AIS_dataframe_wt.sort_values('WaitingTime')

# Appending normalised column
AIS_dataframe_wt['norm'] = norm

# Plot
plt.plot(AIS_dataframe_wt.WaitingTime, AIS_dataframe_wt.norm, color='r')
plt.title('Waiting Time normalised over number of operations')
plt.xlabel("Waiting Time [hours]")
plt.ylabel("Percentage")
#plt.legend()
plt.xlim(0,700)
plt.show()


# ------------------Normalised Post Bunkering Time (PBT) ---------------------

AIS_dataframe_pbt = AIS_dataframe

# Sort by increasing certainty
AIS_dataframe_pbt = AIS_dataframe_pbt.sort_values('PBTime')

# Append normalised column
AIS_dataframe_pbt['norm'] = norm

# Plot
plt.plot(AIS_dataframe_pbt.PBTime, AIS_dataframe_pbt.norm)
plt.title('Post Bunkering Time normalised over number of operations')
plt.xlabel("Post Bunkering Time [hours]")
plt.ylabel("Percentage")
#plt.legend()
plt.xlim(0,700)
plt.show()


# ------------------ Normalised WT and PBT ---------------------

plt.plot(AIS_dataframe_pbt.PBTime, AIS_dataframe_pbt.norm, label="PBT", color = "b")
plt.plot(AIS_dataframe_wt.WaitingTime, AIS_dataframe_wt.norm, label="WT", color = "r")
plt.title('WT and PBT normalised over number of operations')
plt.xlabel("Time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,200)
plt.legend()
plt.show()

# Mark 75% quantile
plt.plot(AIS_dataframe_pbt.PBTime, AIS_dataframe_pbt.norm, label="PBT", color = "b")
plt.plot(AIS_dataframe_wt.WaitingTime, AIS_dataframe_wt.norm, label="WT", color = "r")
plt.axhline(y=75, color="grey")
plt.axhline(y=25, color="grey")
plt.plot(5.57,75, 'o', color='grey')
plt.plot(10.58,75, 'o', color='grey')
plt.plot([5.55,5.55],[0,75], '-', color='grey')
plt.plot([10.58,10.58],[0,75], '-', color='grey')
plt.text(3.5, 77, 'x = 5.55')
plt.text(11, 70, 'x = 10.58')
plt.text(32, 76, 'y = 75', color='grey')
plt.plot(1,25, 'o', color='grey')
plt.plot(2.1,25, 'o', color='grey')
plt.plot([1,1],[0,25], '-', color='grey')
plt.plot([2.1,2.1],[0,25], '-', color='grey')
plt.text(0.1, 26, 'x = 1')
plt.text(2.5, 26, 'x = 2.1')
plt.text(32, 26, 'y = 25', color='grey')
plt.title('WT and PBT normalised over number of operations')
plt.xlabel("Time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,35)
plt.ylim(0,105)
plt.legend()
plt.show()


