"""
Plot distribution of anchorage and bunkering time lengths for ships,
and truncated plots normalised over number of operations.
Created: 13. April 2018
Author: Marion Aarsnes
"""

import pandas as pd                     # Read CSV
import seaborn as sns                   # Library of plots built upon matplotlib
import matplotlib.pyplot as plt         # Plots
import matplotlib.ticker as mtick       # Formatting plots
import numpy as np                      # Scientific computing
from matplotlib.pyplot import xticks    # Get or set the current tick locations and labels of the x-axis

# Read CSV file
AIS_dataframe = pd.read_csv('filedirectory/filename.csv')

# Clean up dataframe
AIS_dataframe = AIS_dataframe[['imo.b', 'shipType.s', 'length.s', 'bunkeringLength_hrs', 'anchoringLength_hrs', 'bunkeringType', 'confidence_mean']]
AIS_dataframe = AIS_dataframe.rename(columns={"imo.b": "IMO",
                                              "shipType.s": "ShipType",
                                              "length.s": "ShipLength",
                                              "anchoringLength_hrs": "AnchoringLengthHrs",
                                              "bunkeringLength_hrs": "BunkeringLengthHrs",
                                              "bunkeringType": "BunkeringType",
                                              "confidence_mean": "ConfMean"})

AIS_dataframe = AIS_dataframe.fillna(value="nan")[AIS_dataframe.ShipType != "nan"]
# Require a minimum confidence mean of 0.7
AIS_dataframe = AIS_dataframe[AIS_dataframe.ConfMean >= 0.7]
print(AIS_dataframe['ShipType'].unique())

# Reduce dataframe to shiptypes with minimum 100 registrations
AIS_dataframe = AIS_dataframe.query("ShipType in ('Bulk Carrier','Container Ship','Crude Oil Tanker','Chemical/Oil Products Tanker',"
                                    "'Vehicles Carrier','LPG Tanker','General Cargo Ship','Oil Products Tanker',"
                                    "'Ore Carrier','LNG Tanker',)")

# Set plot style
sns.set(style="whitegrid", color_codes=True)


# -------------------- Anchorage lasting longer than value a -----------------------

# Create new dataframe for longer lasting anchorages
a = 150
b = 200 # Not used

# Create one value for all operations lasting longer than value a
def fill_values(a):
    anch_len_x = []
    bunk_len_x = []
    for i in range(len(AIS_dataframe.AnchoringLengthHrs)):
        anch_len_x.append(a)
        bunk_len_x.append(b)
    return anch_len_x, bunk_len_x

anch_len_x, bunk_len_x = fill_values(a)

# Create column index to dataframe
AIS_dataframe['AnchorLengthx'] = AIS_dataframe.index
AIS_dataframe['BunkLengthx'] = AIS_dataframe.index
# Append new data to dataframe
AIS_dataframe.loc[:,'AnchorLengthx'] = pd.Series(anch_len_x, index=AIS_dataframe.index)
AIS_dataframe.loc[:,'BunkLengthx'] = pd.Series(bunk_len_x, index=AIS_dataframe.index)

# ------------------------------- Length grouping ---------------------------------

# Create new dataframe for operations lasting longer than value a
AIS_dataframe_x = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs >= a]

# Create new dataframe for operations lasting less than value a
AIS_dataframe_n = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs < a]


# ------------------ Lay time for all ships --------------------

# Define number of bins
# Bin size
res_all = 4

# Calculate bin number based on bin size
min_as = min(AIS_dataframe_n['AnchoringLengthHrs'])
max_as = max(AIS_dataframe_n['AnchoringLengthHrs'])
bsize_as = int((max_as-min_as)/res_all)


# Plot histogram
plt.figure()
ax3 = sns.distplot(AIS_dataframe_n['AnchoringLengthHrs'],
                     bins=bsize_as, hist_kws=dict(edgecolor="b", linewidth=1), label='Lay time < 150 hours')
axb = sns.distplot(AIS_dataframe_x['AnchorLengthx'],
                     kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Lay time > 150 hours')
ax3.set(ylabel="Density", xlabel="Lay time [hours]", title ='All ships')
plt.xlim(0,160)
plt.legend()
plt.show()

# Laying length normalised over number of operations
# Create new column with normalised values
def fill_value_conf():
    norm = []
    for i in range(len(AIS_dataframe_n.AnchoringLengthHrs)):
        norm.append((i+1)/len(AIS_dataframe_n.AnchoringLengthHrs)*100)
    return norm

norm = fill_value_conf()

# Sort by increasing certainty
AIS_dataframe_sortlen = AIS_dataframe_n.sort_values('AnchoringLengthHrs')

# Append normalised column
AIS_dataframe_sortlen['norm'] = norm

# Plot
plt.plot(AIS_dataframe_sortlen.AnchoringLengthHrs, AIS_dataframe_sortlen.norm)
plt.title('Laying length normalised over number of operations')
plt.xlabel("Lay time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,160)
plt.show()



# ------------------ Laying time for anchorage and berth --------------------

# Create new dataframes for bunkering type
# Anchorage
AIS_dataframe_anchor = AIS_dataframe_n[AIS_dataframe_n.BunkeringType == 'Anchorage']
# Berth
AIS_dataframe_berth = AIS_dataframe_n[AIS_dataframe_n.BunkeringType == 'Berth']

# Define number of bins
# Bin size
res_anch = 4 # bin size at anchorage
res_bunk = 1 # bin size at berth

# Calculate bin number based on bin size for anchorage
min_asm = min(AIS_dataframe_anchor['AnchoringLengthHrs'])
max_asm = max(AIS_dataframe_anchor['AnchoringLengthHrs'])
bsize_asm = int((max_asm-min_asm)/res_anch)

# Calculate bin number based on bin size for berth
min_bsm = min(AIS_dataframe_berth['AnchoringLengthHrs'])
max_bsm = max(AIS_dataframe_berth['AnchoringLengthHrs'])
bsize_bsm = int((max_bsm-min_bsm)/res_anch)

# ----------------- Plot time at anchor/berth for all ships --------------------

# ----------------- Anchorage -----------------
# Plot histogram for time at anchorage
plt.figure()
ax = sns.distplot(AIS_dataframe_anchor['AnchoringLengthHrs'],
                     bins=bsize_asm, hist_kws=dict(edgecolor="b", linewidth=1), label='Anchorage time < 150 hours')
axa = sns.distplot(AIS_dataframe_x[AIS_dataframe_x.BunkeringType == 'Anchorage']['AnchorLengthx'],
                     kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Anchorage time > 150 hours')
ax.set(ylabel="Density", xlabel="Anchorage time [hours]", title ='At anchorage')
plt.xlim(0,160)
plt.legend()
plt.show()

# Normalised laying time at anchorage

# Deine new dataframe
AIS_dataframe_anchor_len = AIS_dataframe_anchor

# Sort by increasing certainty
AIS_dataframe_anchor_len = AIS_dataframe_anchor_len.sort_values('AnchoringLengthHrs')

# Create new column with normalised values
def fill_value_anch():
    norm_anch = []
    for i in range(len(AIS_dataframe_anchor)):
        norm_anch.append((i+1)/len(AIS_dataframe_anchor.AnchoringLengthHrs)*100)
    return norm_anch

norm_anch = fill_value_anch()

# Append new column to dataframe
AIS_dataframe_anchor_len['norm'] = norm_anch

# Plot
plt.plot(AIS_dataframe_anchor_len.AnchoringLengthHrs, AIS_dataframe_anchor_len.norm)
plt.title('Anchoring Length at Anchorage normalised over number of operations')
plt.xlabel("Anchorage time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,160)
plt.show()

# ----------------- Berth -----------------
# Plot histogram for time at berth
plt.figure()
ax3 = sns.distplot(AIS_dataframe_berth['AnchoringLengthHrs'],
                     bins=bsize_bsm, hist_kws=dict(edgecolor="b", linewidth=1), label='Berth time < 150 hours')
axb = sns.distplot(AIS_dataframe_x[AIS_dataframe_x.BunkeringType == 'Berth']['AnchorLengthx'],
                     kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Berth time > 150 hours')
ax3.set(ylabel="Density", xlabel="Berth time [hours]", title ='At berth')
plt.xlim(0,160)
plt.legend()
plt.show()

# Normalised lay time at berth

# Create new dataframe
AIS_dataframe_berth_len = AIS_dataframe_berth

# Sort by increasing certainty
AIS_dataframe_berth_len = AIS_dataframe_berth_len.sort_values('AnchoringLengthHrs')

# Create new column with normalised values
def fill_value_berth():
    norm_berth = []
    for i in range(len(AIS_dataframe_berth)):
        norm_berth.append((i+1)/len(AIS_dataframe_berth.AnchoringLengthHrs)*100)
    return norm_berth

norm_berth = fill_value_berth()

# Append new column to dataframe
AIS_dataframe_berth_len['norm'] = norm_berth

# Plot
plt.plot(AIS_dataframe_berth_len.AnchoringLengthHrs, AIS_dataframe_berth_len.norm)
plt.title('Anchoring Length at Berth normalised over number of operations')
plt.xlabel("Berth time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,160)
plt.show()

# Plot anchorage and berth
plt.plot(AIS_dataframe_berth_len.AnchoringLengthHrs, AIS_dataframe_berth_len.norm, label='Berth time')
plt.plot(AIS_dataframe_anchor_len.AnchoringLengthHrs, AIS_dataframe_anchor_len.norm, '--', color='grey', label='Anchorage time')
plt.title('Anchoring Length at Berth normalised over number of operations')
plt.xlabel("Berth time [hours]")
plt.ylabel("Percentage")
plt.xlim(0,160)
plt.legend()
plt.show()
