"""
Plot distribution of anchorage and bunkering time lengths for all operations,
and truncated plots normalised over number of operations.
Created: 13. April 2018
Author: Marion Aarsnes
"""

import pandas as pd                     # Read CSV
import seaborn as sns                   # Library of plots built upon matplotlib
import matplotlib.pyplot as plt         # Plots
import matplotlib.ticker as mtick       # Formatting plots
import numpy as np                      # Scientific computing
from matplotlib.pyplot import xticks    # Get or set the current tick locations and labels of the x-axis

# Read CSV file
AIS_dataframe = pd.read_csv('filedirectory/filename.csv')

# Clean up dataframe
AIS_dataframe = AIS_dataframe[['imo.b', 'shipType.s', 'length.s', 'bunkeringLength_hrs', 'anchoringLength_hrs', 'bunkeringType', 'confidence_mean']]
AIS_dataframe = AIS_dataframe.rename(columns={"imo.b": "IMO",
                                              "shipType.s": "ShipType",
                                              "length.s": "ShipLength",
                                              "anchoringLength_hrs": "AnchoringLengthHrs",
                                              "bunkeringLength_hrs": "BunkeringLengthHrs",
                                              "bunkeringType": "BunkeringType",
                                              "confidence_mean": "ConfMean"})

AIS_dataframe = AIS_dataframe.fillna(value="nan")[AIS_dataframe.ShipType != "nan"]
# Require a minimum confidence mean of 0.7
AIS_dataframe = AIS_dataframe[AIS_dataframe.ConfMean >= 0.7]
print(AIS_dataframe['ShipType'].unique())

# Reduce dataframe to shiptypes with minimum 100 registrations
AIS_dataframe = AIS_dataframe.query("ShipType in ('Bulk Carrier','Container Ship','Crude Oil Tanker','Chemical/Oil Products Tanker',"
                                    "'Vehicles Carrier','LPG Tanker','General Cargo Ship','Oil Products Tanker',"
                                    "'Ore Carrier','LNG Tanker',)")

# Set plot style
sns.set(style="whitegrid", color_codes=True)

# -------------------- Anchorage lasting longer than value a -----------------------

# Create new dataframe for longer lasting anchorages
a = 150
b = 200 # Not used

# Create one value for all operations lasting longer than value a
def fill_values(a):
    anch_len_x = []
    bunk_len_x = []
    for i in range(len(AIS_dataframe.AnchoringLengthHrs)):
        anch_len_x.append(a)
        bunk_len_x.append(b)
    return anch_len_x, bunk_len_x

anch_len_x, bunk_len_x = fill_values(a)

# Create column index to dataframe
AIS_dataframe['AnchorLengthx'] = AIS_dataframe.index
AIS_dataframe['BunkLengthx'] = AIS_dataframe.index
# Append new data to dataframe
AIS_dataframe.loc[:,'AnchorLengthx'] = pd.Series(anch_len_x, index=AIS_dataframe.index)
AIS_dataframe.loc[:,'BunkLengthx'] = pd.Series(bunk_len_x, index=AIS_dataframe.index)



# ------------------------------- Create new dataframes ---------------------------------

# Create new dataframe for operations lasting longer than value a
AIS_dataframe_x = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs >= a]

# Create new dataframe for operations lasting less than value a
AIS_dataframe_n = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs < a]

# Create new dataframes for bunkering type
# Anchorage
AIS_dataframe_anchor = AIS_dataframe_n[AIS_dataframe_n.BunkeringType == 'Anchorage']
# Berth
AIS_dataframe_berth = AIS_dataframe_n[AIS_dataframe_n.BunkeringType == 'Berth']

# ---------------- Scatter plot -----------------
plt.scatter(AIS_dataframe_n.AnchoringLengthHrs, AIS_dataframe_n.BunkeringLengthHrs)
#plt.xlim(-10,200)
#plt.ylim(-10,200)
plt.title('Lay Time vs Bunkering Time')
plt.ylabel("Bunkering Time [hours]")
plt.xlabel("Lay Time [hours]")
plt.xlim(0,160)
plt.show()

grid = sns.jointplot(AIS_dataframe_n.AnchoringLengthHrs, AIS_dataframe_n.BunkeringLengthHrs, kind="reg", joint_kws={'line_kws':{'color':'y'}})
grid.set_axis_labels("Lay time [hours]","Bunkering time [hours]")
plt.xlim(0,160)
plt.ylim(0,35)
plt.show()

# ------------------- Laying time and bunkering time -----------------------

# Define number of bins
# Bin size
res_all = 4

# Calculate bin number based on bin size
min_as = min(AIS_dataframe_n['AnchoringLengthHrs'])
max_as = max(AIS_dataframe_n['AnchoringLengthHrs'])
bsize_as = int((max_as-min_as)/res_all)

# Define number of bins for bunkering time
# Bin size
res_bll = 4

# Calculate bin number based on bin size
min_bs = min(AIS_dataframe_n['BunkeringLengthHrs'])
max_bs = max(AIS_dataframe_n['BunkeringLengthHrs'])
bsize_bs = int((max_bs-min_bs)/res_bll)


# Plot histogram
plt.figure()
ax3 = sns.distplot(AIS_dataframe_n['AnchoringLengthHrs'],
                     bins=bsize_as, hist_kws=dict(edgecolor="b", linewidth=1), label='Lay time < 150 hours')
ax4 = sns.distplot(AIS_dataframe_n['BunkeringLengthHrs'],
                     bins=bsize_bs, color='r', hist_kws=dict(edgecolor="r", linewidth=1), label='Bunkering time')
ax3.set(ylabel="Density", xlabel="Time [hours]", title ='All ships')
plt.xlim(0,160)
plt.legend()
plt.show()


# ------------------- At anchorage -----------------------

# Define number of bins
# Bin size
res_anch = 4 # bin size at anchorage

# Calculate bin number based on bin size for anchorage
min_anch = min(AIS_dataframe_anchor['AnchoringLengthHrs'])
max_anch = max(AIS_dataframe_anchor['AnchoringLengthHrs'])
bsize_anch = int((max_anch-min_anch)/res_anch)

min_abunk = min(AIS_dataframe_anchor['BunkeringLengthHrs'])
max_abunk = max(AIS_dataframe_anchor['BunkeringLengthHrs'])
bsize_abunk = int((max_abunk-min_abunk)/res_anch)


# Plot histogram
plt.figure()
ax3 = sns.distplot(AIS_dataframe_anchor['AnchoringLengthHrs'],
                     bins=bsize_anch, hist_kws=dict(edgecolor="b", linewidth=1), label='Anchorage time < 150 hours')
ax4 = ax3 = sns.distplot(AIS_dataframe_anchor['BunkeringLengthHrs'],
                     bins=bsize_abunk, color='r', hist_kws=dict(edgecolor="r", linewidth=1), label='Bunkering time')
ax3.set(ylabel="Density", xlabel="Time [hours]", title ='At anchorage')
plt.xlim(0,160)
plt.legend()
plt.show()


# ------------------- At berth -----------------------

# Define number of bins
# Bin size
res_ber = 4 # bin size at berth

# Calculate bin number based on bin size for berth
min_ber = min(AIS_dataframe_berth['AnchoringLengthHrs'])
max_ber = max(AIS_dataframe_berth['AnchoringLengthHrs'])
bsize_ber = int((max_ber-min_ber)/res_ber)

min_bbunk = min(AIS_dataframe_berth['BunkeringLengthHrs'])
max_bbunk = max(AIS_dataframe_berth['BunkeringLengthHrs'])
bsize_bbunk = int((max_bbunk-min_bbunk)/res_ber)

# Plot histogram
plt.figure()
ax3 = sns.distplot(AIS_dataframe_berth['AnchoringLengthHrs'],
                     bins=bsize_ber, hist_kws=dict(edgecolor="b", linewidth=1), label='Berth time < 150 hours')
ax4 = ax3 = sns.distplot(AIS_dataframe_berth['BunkeringLengthHrs'],
                     bins=bsize_bbunk, color='r', hist_kws=dict(edgecolor="r", linewidth=1), label='Bunkering time')
ax3.set(ylabel="Density", xlabel="Time [hours]", title ='At berth')
plt.xlim(0,160)
plt.legend()
plt.show()




# ----------------------- Time normalised over number of operations ------------------------

# All operations
# Create new column with normalised values
def fill_value_all():
    norm = []
    for i in range(len(AIS_dataframe_n.AnchoringLengthHrs)):
        norm.append((i+1)/len(AIS_dataframe_n.AnchoringLengthHrs)*100)
    return norm

norm = fill_value_all()

# Create dataframe for laying length
AIS_dataframe_norml = AIS_dataframe_n
# Sort by increasing certainty
AIS_dataframe_norml = AIS_dataframe_norml.sort_values('AnchoringLengthHrs')
# Append normalised column
AIS_dataframe_norml['norm'] = norm

# Create dataframe for bunkering length
# Sort by increasing certainty
AIS_dataframe_normb = AIS_dataframe_n.sort_values('BunkeringLengthHrs')
# Append normalised column
AIS_dataframe_normb['norm'] = norm

# Plot
plt.plot(AIS_dataframe_norml.AnchoringLengthHrs, AIS_dataframe_norml.norm, label='Lay time')
plt.plot(AIS_dataframe_normb.BunkeringLengthHrs, AIS_dataframe_normb.norm, label='Bunkering time', color='r')
plt.title('Lay time and bunkering time normalised over number of operations')
plt.xlabel("Time [hours]")
plt.ylabel("Percentage")
plt.legend()
plt.xlim(0,160)
plt.show()

# Anchorage
# Create new column with normalised values
def fill_value_anch():
    norm_anch = []
    for i in range(len(AIS_dataframe_anchor.AnchoringLengthHrs)):
        norm_anch.append((i+1)/len(AIS_dataframe_anchor.AnchoringLengthHrs)*100)
    return norm_anch

norm_anch = fill_value_anch()

# Create dataframe for laying length
AIS_dataframe_normla = AIS_dataframe_anchor
# Sort by increasing certainty
AIS_dataframe_normla = AIS_dataframe_normla.sort_values('AnchoringLengthHrs')

print(len(AIS_dataframe_normla))
print(len(norm_anch))
# Append normalised column
AIS_dataframe_normla['norm_anch'] = norm_anch

# Create dataframe for bunkering length
# Sort by increasing certainty
AIS_dataframe_normbl = AIS_dataframe_anchor.sort_values('BunkeringLengthHrs')
# Append normalised column
AIS_dataframe_normbl['norm_anch'] = norm_anch

# Plot
plt.plot(AIS_dataframe_normla.AnchoringLengthHrs, AIS_dataframe_normla.norm_anch, label='Anchorage time')
plt.plot(AIS_dataframe_normbl.BunkeringLengthHrs, AIS_dataframe_normbl.norm_anch, label='Bunkering time', color='r')
plt.title('Lay time and bunkering time normalised over number of operations at anchorage')
plt.xlabel("Time [hours]")
plt.ylabel("Percentage")
plt.legend()
plt.xlim(0,160)
plt.show()


# Berth
# Create new column with normalised values
def fill_value_ber():
    norm_ber = []
    for i in range(len(AIS_dataframe_berth.AnchoringLengthHrs)):
        norm_ber.append((i+1)/len(AIS_dataframe_berth.AnchoringLengthHrs)*100)
    return norm_ber

norm_ber = fill_value_ber()

# Create dataframe for laying length
AIS_dataframe_normlb = AIS_dataframe_berth
# Sort by increasing certainty
AIS_dataframe_normlb = AIS_dataframe_normlb.sort_values('AnchoringLengthHrs')

print(len(AIS_dataframe_normlb))
print(len(norm_ber))
# Append normalised column
AIS_dataframe_normlb['norm_ber'] = norm_ber

# Create dataframe for bunkering length
# Sort by increasing certainty
AIS_dataframe_normblb = AIS_dataframe_berth.sort_values('BunkeringLengthHrs')
# Append normalised column
AIS_dataframe_normblb['norm_ber'] = norm_ber

# Plot
plt.plot(AIS_dataframe_normlb.AnchoringLengthHrs, AIS_dataframe_normlb.norm_ber, label='Berth time')
plt.plot(AIS_dataframe_normblb.BunkeringLengthHrs, AIS_dataframe_normblb.norm_ber, label='Bunkering time', color='r')
plt.title('Berth time and bunkering time normalised over number of operations at berth')
plt.xlabel("Time [hours]")
plt.ylabel("Percentage")
plt.legend()
plt.xlim(0,160)
plt.show()

# -------------------- Less than 60 hours lay time -----------------

AIS_dataframe_nn = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs<60]

# Define number of bins
# Bin size
res_nn = 4

# Calculate bin number based on bin size
min_nn = min(AIS_dataframe_nn['AnchoringLengthHrs'])
max_nn = max(AIS_dataframe_nn['AnchoringLengthHrs'])
bsize_nn = int((max_nn-min_nn)/res_nn)

# Define number of bins
# Bin size
res_bnn = 4

# Calculate bin number based on bin size
min_bnn = min(AIS_dataframe_nn['BunkeringLengthHrs'])
max_bnn = max(AIS_dataframe_nn['BunkeringLengthHrs'])
bsize_bnn = int((max_bnn-min_bnn)/res_bnn)


# Plot histogram
plt.figure()
ax3 = sns.distplot(AIS_dataframe_nn['AnchoringLengthHrs'],
                     bins=bsize_nn, hist_kws=dict(edgecolor="b", linewidth=1), label='Lay time < 60 hours')
ax4 = sns.distplot(AIS_dataframe_nn['BunkeringLengthHrs'],
                     bins=bsize_bnn, color='r', hist_kws=dict(edgecolor="r", linewidth=1), label='Bunkering time')
ax3.set(ylabel="Density", xlabel="Time [hours]", title ='All ships')
plt.xlim(0,160)
plt.legend()
plt.show()


# Normalised graphs
# Berth
# Create new column with normalised values
def fill_value_nn():
    norm_nn = []
    for i in range(len(AIS_dataframe_nn.AnchoringLengthHrs)):
        norm_nn.append((i+1)/len(AIS_dataframe_nn.AnchoringLengthHrs)*100)
    return norm_nn

norm_nn = fill_value_nn()

# Sort by increasing increasing lay time
AIS_dataframe_nnlay = AIS_dataframe_nn.sort_values('AnchoringLengthHrs')
# Sort by increasing increasing bunkering time
AIS_dataframe_nnbun = AIS_dataframe_nn.sort_values('BunkeringLengthHrs')

# Append normalised column
AIS_dataframe_nnlay['norm_nn'] = norm_nn
AIS_dataframe_nnbun['norm_nn'] = norm_nn

# Scatter plot
plt.plot(AIS_dataframe_nnlay.AnchoringLengthHrs, AIS_dataframe_nnlay.norm_nn, label='Lay time')
plt.plot(AIS_dataframe_nnbun.BunkeringLengthHrs, AIS_dataframe_nnbun.norm_nn, label='Bunkering time', color='r')
plt.title('Lay time and bunkering time normalised over number of operations')
plt.xlabel("Time [hours]")
plt.ylabel("Percentage")
plt.legend()
plt.xlim(0,160)
plt.show()



