"""
Plot anchoring and bunkering for several barges in one plot
Created: 12. April 2018
Author: Marion Aarsnes
"""

# Import packages
import pandas as pd                 # Read CSV
import numpy as np                  # Scientific computation
import matplotlib.pyplot as plt     # Plotting
import matplotlib.ticker as mtick   # Formatting plots
import seaborn as sns               # Library of plots built upon matplotlib

# Read CSV file
AIS_dataframe = pd.read_csv('filedirectory/filename.csv')
print(AIS_dataframe.columns)

# Clean up dataframe
AIS_dataframe = AIS_dataframe[['imo.b', 'shipType.b', 'length.b', 'shipType.s', 'name.s', 'length.s', 'bunkeringLength_hrs', 'anchoringLength_hrs', 'BunkUnixStart',
       'BunkUnixEnd', 'AnchorUnixStart', 'AnchorUnixEnd']]
AIS_dataframe = AIS_dataframe.rename(columns={"imo.b": "IMO",
                                              "shipType.b": "BargeType",
                                              "length.b": "BargeLength",
                                              "shipType.s": "ShipType",
                                              "length.s": "ShipLength",
                                              "name.s": "ShipName",
                                              "anchoringLength_hrs": "AnchoringLengthHrs",
                                              "bunkeringLength_hrs": "BunkeringLengthHrs"})

AIS_dataframe = AIS_dataframe.fillna(value="nan")[AIS_dataframe.ShipType != "nan"]

# Set plot variables
sns.set(style="whitegrid", color_codes=True)
sns.despine(left=True)

# Define operation numbers to plot
op_list = [3, 4, 10]

# Create empty matrices for x and y values
x_list = np.zeros(shape=(len(op_list),4))
y_list = np.zeros(shape=(len(op_list),2))

# Calculate matrix values - start and end of anchorage and bunkering for each operation
for j in range(len(op_list)):
        i = op_list[j]
        x_list[j,0] = (AIS_dataframe.AnchorUnixStart[i] - AIS_dataframe.AnchorUnixStart[i])/3600
        x_list[j,1] = (AIS_dataframe.AnchorUnixEnd[i] - AIS_dataframe.AnchorUnixStart[i])/3600
        x_list[j,2] = (AIS_dataframe.BunkUnixStart[i] - AIS_dataframe.AnchorUnixStart[i])/3600
        x_list[j,3] = (AIS_dataframe.BunkUnixEnd[i] - AIS_dataframe.AnchorUnixStart[i])/3600
        y_list[j,0] = j
        y_list[j,1] = j+0.1

# Define colors and line styles
colors = ['r', 'b', 'y', 'g', 'm', 'k']
linestyle = ['-', '--', ':', '-.']

# Plot
for k in range(len(op_list)):
    type_s = AIS_dataframe.ShipType[op_list[k]]
    length_s = AIS_dataframe.ShipLength[op_list[k]]
    type_b = AIS_dataframe.BargeType[op_list[k]]
    length_b = AIS_dataframe.BargeLength[op_list[k]]
    plt.plot([x_list[k,0], x_list[k,1]], [y_list[k,0], y_list[k,0]], '1', color='b')
    plt.plot([x_list[k,2], x_list[k,3]], [y_list[k,1], y_list[k,1]], '1', color='r')
    plt.plot([x_list[k,0], x_list[k,1]], [y_list[k,0], y_list[k,0]], linestyle[k], color='b', label='Anchorage ' + str(type_s) + ' ' + str(length_s) + 'm')
    plt.plot([x_list[k,2], x_list[k,3]], [y_list[k,1], y_list[k,1]], linestyle[k], color='r', label='Bunkering ' + str(type_b) + ' ' + str(length_b) + 'm')
    plt.plot([x_list[k,0], x_list[k,0]], [y_list[k,0]+0.1, y_list[k,0]-0.1], '-', color='b')
    plt.plot([x_list[k,1], x_list[k,1]], [y_list[k,0]+0.1, y_list[k,0]-0.1], '-', color='b')
    plt.plot([x_list[k, 2], x_list[k, 2]], [y_list[k, 1] + 0.1, y_list[k, 0]], '-', color='r')
    plt.plot([x_list[k, 3], x_list[k, 3]], [y_list[k, 1] + 0.1, y_list[k, 0]], '-', color='r')
    plt.text(9.9, 2.2, 'Operation 3')
    plt.text(14, 1.2, 'Operation 2')
    plt.text(3.5, 0.2, 'Operation 1')

plt.title('Time plot anchorage and bunkering')
plt.xlabel("Time [hours]")
plt.legend()
plt.show()